﻿using System;
using System.Collections.Generic;
using System.Text;
using System.ComponentModel.Design.Serialization;
using System.ComponentModel.Design;
using System.Collections;
using System.Drawing;
using System.Xml;
using System.Windows.Forms;
using DxDesigner.Components;

namespace DxDesigner.Host
{
    /// <summary>
    /// Service responsible for loading design from XML source file and saving to XML format.
    /// Additionally detect changes in desing (dirty detection)
    /// </summary>
    class DxDesignerLoader:BasicDesignerLoader
    {
        public DxDesignerLoader()
		{
		}

        protected override void PerformLoad(IDesignerSerializationManager designerSerializationManager)
        {
            //LoaderHost.CreateComponent(typeof(DxContainer));

            ReadFile();


            // Now that we are done with the load work, we need to begin to listen to events.
            // Listening to event notifications is how a designer "Loader" can also be used
            // to save data.  If we wanted to integrate this loader with source code control,
            // we would listen to the "ing" events as well as the "ed" events.

            IComponentChangeService cs = LoaderHost.GetService(typeof(IComponentChangeService)) as IComponentChangeService;

            if (cs != null)
            {
                cs.ComponentChanged += new ComponentChangedEventHandler(OnComponentChanged);
                cs.ComponentAdded += new ComponentEventHandler(OnComponentAddedRemoved);
                cs.ComponentRemoved += new ComponentEventHandler(OnComponentAddedRemoved);
            }

            // Let the host know we are done loading.
            ArrayList errors = new ArrayList();
            bool successful = true;
            LoaderHost.EndLoad("DxContainer", successful, errors);
        }

        protected override void PerformFlush(IDesignerSerializationManager designerSerializationManager)
        {
            // Nothing to flush if nothing has changed.
            //if (!Modified)
            //{
            //    return;
           //}

            XmlDocument doc = new XmlDocument();
            XmlNode rootnode = doc.CreateElement("dxqsl");
            doc.AppendChild(rootnode);
            IDesignerHost dh = (IDesignerHost)GetService(typeof(IDesignerHost));
            DxContainer cont = (DxContainer)(dh.RootComponent);
            XmlNode contnode = cont.DxSerialize(doc);
            
            foreach (Control ctrl in cont.Controls)
            {
                XmlNode compnode = null;
                if (ctrl is DxLabel)
                    compnode = ((DxLabel)ctrl).DxSerialize(doc);
                else if (ctrl is DxSwlCall)
                    compnode = ((DxSwlCall)ctrl).DxSerialize(doc);
                else if (ctrl is DxText)
                    compnode = ((DxText)ctrl).DxSerialize(doc);
                else if (ctrl is DxDateTime)
                    compnode = ((DxDateTime)ctrl).DxSerialize(doc);
                else if (ctrl is DxNumber)
                    compnode = ((DxNumber)ctrl).DxSerialize(doc);
                else if (ctrl is DxPicture)
                    compnode = ((DxPicture)ctrl).DxSerialize(doc);
                else if (ctrl is DxFrame)
                    compnode = ((DxFrame)ctrl).DxSerialize(doc);
                if (compnode != null)
                    contnode.AppendChild(compnode);
            }
            rootnode.AppendChild(contnode);
            doc.Save(MainForm.fileName);
        }


        /// <summary>
        /// Create all design components based on XML document
        /// </summary>
        private void ReadFile()
        {
            try
            {
                if (MainForm.fileName == "")
                {
                    DxContainer container = (DxContainer)LoaderHost.CreateComponent(typeof(DxContainer));
                    container.DxDeserialize(null);
                }
                else
                {

                    XmlDocument doc = new XmlDocument();
                    doc.Load(MainForm.fileName);

                    XmlNode el = doc.FirstChild;
                    if (el.Name != "dxqsl")
                        throw new Exception("Not a DxQSL template!");
                    el = el.FirstChild;
                    if (el.Name != "template")
                        throw new Exception("Error DxQSL template!");

                    DxContainer container = (DxContainer)LoaderHost.CreateComponent(typeof(DxContainer));
                    container.DxDeserialize(el.FirstChild);

                    XmlNode templComp = el.FirstChild;
                    while (templComp != null)
                    {

                        switch (templComp.Name.ToLower())
                        {
                            case "label":
                                DxLabel label = (DxLabel)LoaderHost.CreateComponent(typeof(DxLabel));
                                label.DxDeserialize(templComp.FirstChild);
                                label.Parent = container;
                                break;
                            case "swlcall":
                                DxSwlCall swlcall = (DxSwlCall)LoaderHost.CreateComponent(typeof(DxSwlCall));
                                swlcall.DxDeserialize(templComp.FirstChild);
                                swlcall.Parent = container;
                                break;
                            case "text":
                                DxText dxtext = (DxText)LoaderHost.CreateComponent(typeof(DxText));
                                dxtext.DxDeserialize(templComp.FirstChild);
                                dxtext.Parent = container;
                                break;
                            case "datetime":
                                DxDateTime datetime = (DxDateTime)LoaderHost.CreateComponent(typeof(DxDateTime));
                                datetime.DxDeserialize(templComp.FirstChild);
                                datetime.Parent = container;
                                break;
                            case "number":
                                DxNumber dxnumber = (DxNumber)LoaderHost.CreateComponent(typeof(DxNumber));
                                dxnumber.DxDeserialize(templComp.FirstChild);
                                dxnumber.Parent = container;
                                break;
                            case "picture":
                                DxPicture picture = (DxPicture)LoaderHost.CreateComponent(typeof(DxPicture));
                                picture.DxDeserialize(templComp.FirstChild);
                                picture.Parent = container;
                                break;
                        }
                        templComp = templComp.NextSibling;
                    }

                    // insert frame as the last component (it is put to very back` it is not transparent!)
                    templComp = el.FirstChild;
                    while (templComp != null)
                    {

                        if (templComp.Name.ToLower() == "frame")
                        {
                            DxFrame frame = (DxFrame)LoaderHost.CreateComponent(typeof(DxFrame));
                            frame.DxDeserialize(templComp.FirstChild);
                            frame.Parent = container;
                        }
                        templComp = templComp.NextSibling;
                    }
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message,"Error");
            }
        }     

        private void OnComponentChanged(object sender, ComponentChangedEventArgs ce)
        {
            Modified = true;
            DxDateTime dxb = ce.Component as DxDateTime;
            if (dxb != null)
                dxb.Text = dxb.testDate.ToString(dxb.DTFormat);
        }

        private void OnComponentAddedRemoved(object sender, ComponentEventArgs ce)
        {
            Modified = true;
        }

        public bool _Modified
        {
            get
            {
                return Modified;
            }
            set
            {
                Modified = true;
            }
        }
    }
}
